# -*- coding: utf-8 -*-


import jsonfield.fields

from django.db import migrations, models

import enterprise.constants


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0030_auto_20171005_1600'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomercatalog',
            name='enabled_course_modes',
            field=jsonfield.fields.JSONField(default=enterprise.constants.COURSE_MODE_SORT_ORDER, help_text='Ordered list of enrollment modes which can be displayed to learners for course runs in this catalog.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomercatalog',
            name='enabled_course_modes',
            field=jsonfield.fields.JSONField(default=enterprise.constants.COURSE_MODE_SORT_ORDER, help_text='Ordered list of enrollment modes which can be displayed to learners for course runs in this catalog.'),
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-07 11:38

from django.db import migrations

from enterprise.constants import SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE


def create_enterprise_catalog_admin_role(apps, schema_editor):
    """Create the `enterprise_catalog_admin` system-wide role if it does not already exist"""
    SystemWideEnterpriseRole = apps.get_model('enterprise', 'SystemWideEnterpriseRole')
    SystemWideEnterpriseRole.objects.update_or_create(
        name=SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE,
        description='Role for access to endpoints in the enterprise catalog service',
    )


def delete_enterprise_catalog_admin_role(apps, schema_editor):
    """Delete the `enterprise_catalog_admin` system-wide role"""
    SystemWideEnterpriseRole = apps.get_model('enterprise', 'SystemWideEnterpriseRole')
    SystemWideEnterpriseRole.objects.filter(name=SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0095_auto_20200507_1138'),
    ]

    operations = [
        migrations.RunPython(
            code=create_enterprise_catalog_admin_role,
            reverse_code=delete_enterprise_catalog_admin_role,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-12-13 15:23
# This is necessary because django-simple-history 2.6.0 changes the model


import django.db.models.deletion
from django.db import migrations, models

import enterprise.models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0059_add_code_management_portal_config'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalenterprisecourseenrollment',
            name='enterprise_customer_user',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='The enterprise learner to which this enrollment is attached.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomerUser'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='customer_type',
            field=models.ForeignKey(blank=True, db_constraint=False, default=enterprise.models.get_default_customer_type, help_text='Specifies enterprise customer type.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCustomerType', verbose_name='Customer Type'),
        ),
    ]



"""
Module provides elements to be used in third-party auth pipeline.
"""

import re

from django.urls import reverse

from enterprise.models import EnterpriseCustomer, EnterpriseCustomerUser
from enterprise.utils import get_identity_provider

try:
    from social_core.pipeline.partial import partial
except ImportError:
    from enterprise.decorators import null_decorator as partial  # pylint:disable=ungrouped-imports

try:
    from social_django.models import UserSocialAuth
except ImportError:
    UserSocialAuth = None

try:
    from third_party_auth.provider import Registry
except ImportError:
    Registry = None

try:
    from openedx.core.djangoapps.user_api.accounts.utils import is_multiple_user_enterprises_feature_enabled
except ImportError:
    is_multiple_user_enterprises_feature_enabled = None


def get_enterprise_customer_for_running_pipeline(request, pipeline):  # pylint: disable=invalid-name
    """
    Get the EnterpriseCustomer associated with a running pipeline.
    """
    sso_provider_id = request.GET.get('tpa_hint')
    if pipeline:
        sso_provider_id = Registry.get_from_pipeline(pipeline).provider_id
    return get_enterprise_customer_for_sso(sso_provider_id)


def get_enterprise_customer_for_sso(sso_provider_id):
    """
    Get the EnterpriseCustomer object tied to an identity provider.
    """
    try:
        return EnterpriseCustomer.objects.get(  # pylint: disable=no-member
            enterprise_customer_identity_provider__provider_id=sso_provider_id
        )
    except EnterpriseCustomer.DoesNotExist:
        return None


@partial
def handle_enterprise_logistration(backend, user, **kwargs):
    """
    Perform the linking of user in the process of logging to the Enterprise Customer.

    Args:
        backend: The class handling the SSO interaction (SAML, OAuth, etc)
        user: The user object in the process of being logged in with
        **kwargs: Any remaining pipeline variables

    """
    request = backend.strategy.request
    enterprise_customer = get_enterprise_customer_for_running_pipeline(
        request,
        {
            'backend': backend.name,
            'kwargs': kwargs
        }
    )
    if enterprise_customer is None:
        # This pipeline element is not being activated as a part of an Enterprise logistration

        # Social user account is new or already binded with edx account
        new_association = kwargs.get('new_association', True)
        if is_multiple_user_enterprises_feature_enabled() and not new_association:
            handle_redirect_after_social_auth_login(backend, user)
        return

    # proceed with the creation of a link between the user and the enterprise customer, then exit.
    defaults = {'active': True}
    enterprise_customer_user, _ = EnterpriseCustomerUser.objects.update_or_create(
        enterprise_customer=enterprise_customer,
        user_id=user.id,
        defaults=defaults,
    )
    # if learner has activated enterprise we need to de-activate other enterprises learner is linked to
    EnterpriseCustomerUser.objects.filter(
        user_id=user.id
    ).exclude(enterprise_customer=enterprise_customer).update(active=False)
    enterprise_customer_user.update_session(request)


def get_user_from_social_auth(tpa_provider, user_id):
    """
    Find the LMS user from the LMS model `UserSocialAuth`.

    Arguments:
        tpa_provider (third_party_auth.provider): third party auth provider object
        user_id (str): User id of user in third party LMS

    """
    provider_slug = tpa_provider.provider_id.strip('saml-')
    social_auth_uid = '{0}:{1}'.format(provider_slug, user_id)
    # we are filtering by both `provider` and `uid` to make use of provider,uid composite index
    # filtering only on `uid` makes query extremely slow since we don't have index on `uid`
    user_social_auth = UserSocialAuth.objects.select_related('user').filter(
        provider=tpa_provider.backend_name, uid=social_auth_uid
    ).first()

    return user_social_auth.user if user_social_auth else None


def get_user_social_auth(user, enterprise_customer):
    """
    Return social auth entry of user for given enterprise.

    Arguments:
        user (User): user object
        enterprise_customer (EnterpriseCustomer): User id of user in third party LMS

    """
    provider_id = enterprise_customer.identity_provider
    tpa_provider = get_identity_provider(provider_id)
    user_social_auth = UserSocialAuth.objects.filter(
        user=user,
        provider=tpa_provider.backend_name
    ).first()

    return user_social_auth


def handle_redirect_after_social_auth_login(backend, user):
    """
    Change the redirect url if user has more than 1 EnterpriseCustomer associations.

    Arguments:
        backend (User): social auth backend object
        user (User): user object

    """
    enterprise_customers_count = EnterpriseCustomerUser.objects.filter(user_id=user.id).count()
    next_url = backend.strategy.session_get('next')
    using_enrollment_url = re.match(r'/enterprise/.*/course/.*/enroll', str(next_url))
    if (enterprise_customers_count > 1) and not using_enrollment_url:
        select_enterprise_page_as_redirect_url(backend.strategy)


def select_enterprise_page_as_redirect_url(strategy):  # pylint: disable=invalid-name
    """
    Change the redirect url for the user to enterprise selection page.
    """
    current_redirect = strategy.session_get('next')
    select_enterprise_page = reverse('enterprise_select_active') + '?success_url=' + current_redirect
    strategy.session_set('next', select_enterprise_page)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-02-24 15:45


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_company_id',
            field=models.CharField(blank=True, max_length=255, verbose_name='SAP Company ID'),
        ),
        migrations.AddField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_user_id',
            field=models.CharField(blank=True, max_length=255, verbose_name='SAP User ID'),
        ),
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_company_id',
            field=models.CharField(blank=True, max_length=255, verbose_name='SAP Company ID'),
        ),
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_user_id',
            field=models.CharField(blank=True, max_length=255, verbose_name='SAP User ID'),
        ),
        migrations.AlterField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_base_url',
            field=models.CharField(max_length=255, verbose_name='SAP Base URL'),
        ),
        migrations.AlterField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='sapsf_base_url',
            field=models.CharField(max_length=255, verbose_name='SAP Base URL'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Django management command for saving EnterpriseCustomerUser models.
"""

import logging

from django.core.management import BaseCommand

from enterprise.models import EnterpriseCustomerUser

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Calls save() on EnterpriseCustomerUser models.

    This is useful for triggering save-related signals causing the
    associated signal receiver functions to fire.
    """
    help = 'Save existing EnterpriseCustomerUser models.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-e',
            '--enterprise_customer_uuid',
            action='store',
            dest='enterprise_customer_uuid',
            default=None,
            help='Run this command for only the given EnterpriseCustomer UUID.'
        )

    def handle(self, *args, **options):
        enterprise_customer_filter = {}
        enterprise_customer_uuid = options.get('enterprise_customer_uuid')
        if enterprise_customer_uuid:
            enterprise_customer_filter['enterprise_customer'] = enterprise_customer_uuid

        count = 0
        for enterprise_customer_user in EnterpriseCustomerUser.objects.filter(**enterprise_customer_filter):
            enterprise_customer_user.save()
            count += 1

        LOGGER.info('%s EnterpriseCustomerUser models saved.', count)



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0036_sftp_reporting_support'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerentitlement',
            name='entitlement_id',
            field=models.PositiveIntegerField(help_text="Enterprise customer's entitlement id for relationship with e-commerce coupon.", unique=True, verbose_name='Seat Entitlement'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerentitlement',
            name='entitlement_id',
            field=models.PositiveIntegerField(help_text="Enterprise customer's entitlement id for relationship with e-commerce coupon.", verbose_name='Seat Entitlement', db_index=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Permission classes for the Consent API.
"""

from rest_framework import permissions

from enterprise.utils import get_request_value


class IsUserInRequest(permissions.BasePermission):
    """
    Permission that checks to see if the request user matches the user indicated in the request body.
    """

    def has_permission(self, request, view):
        return request.user.username == get_request_value(request, 'username', '')


class IsStaffOrUserInRequest(IsUserInRequest):
    """
    Permission that checks to see if the request user is staff or is the user
    indicated in the request body.
    """

    def has_permission(self, request, view):
        if request.user.is_staff:
            return True

        return super(IsStaffOrUserInRequest, self).has_permission(request, view)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-28 17:26


from django.db import migrations, models

import enterprise.validators


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0085_enterprisecustomeruser_linked'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='contact_email',
            field=models.EmailField(blank=True, help_text='Email to be displayed as public point of contact for enterprise.', max_length=254, null=True),
        ),
        migrations.AddField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='banner_background_color',
            field=models.CharField(blank=True, max_length=7, null=True, validators=[enterprise.validators.validate_hex_color]),
        ),
        migrations.AddField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='banner_border_color',
            field=models.CharField(blank=True, max_length=7, null=True, validators=[enterprise.validators.validate_hex_color]),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='contact_email',
            field=models.EmailField(blank=True, help_text='Email to be displayed as public point of contact for enterprise.', max_length=254, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-04-05 11:47


from django.db import migrations, models

import enterprise.models
import enterprise.validators


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0015_auto_20170130_0003'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterprisecustomeruser',
            options={'verbose_name': 'Enterprise Customer Learner', 'verbose_name_plural': 'Enterprise Customer Learners'},
        ),
        migrations.AlterField(
            model_name='enrollmentnotificationemailtemplate',
            name='html_template',
            field=models.TextField(blank=True, help_text='Fill in a standard Django template that, when rendered, produces the email you want sent to newly-enrolled Enterprise Customer learners. The following variables may be available:\n<ul><li>user_name: A human-readable name for the person being emailed. Be sure to handle the case where this is not defined, as it may be missing in some cases. It may also be a username, if the learner hasn\'t configured their "real" name in the system.</li>    <li>organization_name: The name of the organization sponsoring the enrollment.</li>    <li>enrolled_in: Details of the course or program that was enrolled in. It may contain:    <ul><li>name: The name of the enrollable item (e.g., "Demo Course").</li>        <li>url: A link to the homepage of the enrolled-in item.</li>        <li>branding: A custom branding name for the enrolled-in item. For example, the branding of a MicroMasters program would be "MicroMasters".</li>     <li>start: The date the enrolled-in item becomes available. Render this to text using the Django `date` template filter (see <a href="https://docs.djangoproject.com/en/1.8/ref/templates/builtins/#date">the Django documentation</a>).</li><li>type: Whether the enrolled-in item is a course, a program, or something else.</li></ul></ul>'),
        ),
        migrations.AlterField(
            model_name='enrollmentnotificationemailtemplate',
            name='plaintext_template',
            field=models.TextField(blank=True, help_text='Fill in a standard Django template that, when rendered, produces the email you want sent to newly-enrolled Enterprise Customer learners. The following variables may be available:\n<ul><li>user_name: A human-readable name for the person being emailed. Be sure to handle the case where this is not defined, as it may be missing in some cases. It may also be a username, if the learner hasn\'t configured their "real" name in the system.</li>    <li>organization_name: The name of the organization sponsoring the enrollment.</li>    <li>enrolled_in: Details of the course or program that was enrolled in. It may contain:    <ul><li>name: The name of the enrollable item (e.g., "Demo Course").</li>        <li>url: A link to the homepage of the enrolled-in item.</li>        <li>branding: A custom branding name for the enrolled-in item. For example, the branding of a MicroMasters program would be "MicroMasters".</li>     <li>start: The date the enrolled-in item becomes available. Render this to text using the Django `date` template filter (see <a href="https://docs.djangoproject.com/en/1.8/ref/templates/builtins/#date">the Django documentation</a>).</li><li>type: Whether the enrolled-in item is a course, a program, or something else.</li></ul></ul>'),
        ),
        migrations.AlterField(
            model_name='enrollmentnotificationemailtemplate',
            name='subject_line',
            field=models.CharField(blank=True, help_text='Enter a string that can be used to generate a dynamic subject line for notification emails. The placeholder {course_name} will be replaced with the name of the course or program that was enrolled in.', max_length=100),
        ),
        migrations.AlterField(
            model_name='enterprisecourseenrollment',
            name='course_id',
            field=models.CharField(help_text='The ID of the course in which the learner was enrolled.', max_length=255),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=False, help_text='Specifies whether data sharing consent is enabled or disabled for learners signing in through this enterprise customer. If disabled, consent will not be requested, and eligible data will not be shared.'),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enforce_data_sharing_consent',
            field=models.CharField(choices=[('optional', 'Optional'), ('at_login', 'At Login'), ('at_enrollment', 'At Enrollment')], default='optional', help_text='Specifies whether data sharing consent is optional, is required at login, or is required at enrollment.', max_length=25),
        ),
        migrations.AlterField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='logo',
            field=models.ImageField(blank=True, help_text='Logo images must be in .png format.', max_length=255, null=True, upload_to=enterprise.models.logo_path, validators=[enterprise.validators.validate_image_extension, enterprise.validators.validate_image_size]),
        ),
        migrations.AlterField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='html_template',
            field=models.TextField(blank=True, help_text='Fill in a standard Django template that, when rendered, produces the email you want sent to newly-enrolled Enterprise Customer learners. The following variables may be available:\n<ul><li>user_name: A human-readable name for the person being emailed. Be sure to handle the case where this is not defined, as it may be missing in some cases. It may also be a username, if the learner hasn\'t configured their "real" name in the system.</li>    <li>organization_name: The name of the organization sponsoring the enrollment.</li>    <li>enrolled_in: Details of the course or program that was enrolled in. It may contain:    <ul><li>name: The name of the enrollable item (e.g., "Demo Course").</li>        <li>url: A link to the homepage of the enrolled-in item.</li>        <li>branding: A custom branding name for the enrolled-in item. For example, the branding of a MicroMasters program would be "MicroMasters".</li>     <li>start: The date the enrolled-in item becomes available. Render this to text using the Django `date` template filter (see <a href="https://docs.djangoproject.com/en/1.8/ref/templates/builtins/#date">the Django documentation</a>).</li><li>type: Whether the enrolled-in item is a course, a program, or something else.</li></ul></ul>'),
        ),
        migrations.AlterField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='plaintext_template',
            field=models.TextField(blank=True, help_text='Fill in a standard Django template that, when rendered, produces the email you want sent to newly-enrolled Enterprise Customer learners. The following variables may be available:\n<ul><li>user_name: A human-readable name for the person being emailed. Be sure to handle the case where this is not defined, as it may be missing in some cases. It may also be a username, if the learner hasn\'t configured their "real" name in the system.</li>    <li>organization_name: The name of the organization sponsoring the enrollment.</li>    <li>enrolled_in: Details of the course or program that was enrolled in. It may contain:    <ul><li>name: The name of the enrollable item (e.g., "Demo Course").</li>        <li>url: A link to the homepage of the enrolled-in item.</li>        <li>branding: A custom branding name for the enrolled-in item. For example, the branding of a MicroMasters program would be "MicroMasters".</li>     <li>start: The date the enrolled-in item becomes available. Render this to text using the Django `date` template filter (see <a href="https://docs.djangoproject.com/en/1.8/ref/templates/builtins/#date">the Django documentation</a>).</li><li>type: Whether the enrolled-in item is a course, a program, or something else.</li></ul></ul>'),
        ),
        migrations.AlterField(
            model_name='historicalenrollmentnotificationemailtemplate',
            name='subject_line',
            field=models.CharField(blank=True, help_text='Enter a string that can be used to generate a dynamic subject line for notification emails. The placeholder {course_name} will be replaced with the name of the course or program that was enrolled in.', max_length=100),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecourseenrollment',
            name='course_id',
            field=models.CharField(help_text='The ID of the course in which the learner was enrolled.', max_length=255),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=False, help_text='Specifies whether data sharing consent is enabled or disabled for learners signing in through this enterprise customer. If disabled, consent will not be requested, and eligible data will not be shared.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enforce_data_sharing_consent',
            field=models.CharField(choices=[('optional', 'Optional'), ('at_login', 'At Login'), ('at_enrollment', 'At Enrollment')], default='optional', help_text='Specifies whether data sharing consent is optional, is required at login, or is required at enrollment.', max_length=25),
        ),
        migrations.AlterField(
            model_name='historicaluserdatasharingconsentaudit',
            name='state',
            field=models.CharField(choices=[('not_set', 'Not set'), ('enabled', 'Enabled'), ('disabled', 'Disabled')], default='not_set', help_text='Stores whether the learner linked to this model has consented to have their information shared with the linked EnterpriseCustomer.', max_length=8),
        ),
        migrations.AlterField(
            model_name='userdatasharingconsentaudit',
            name='state',
            field=models.CharField(choices=[('not_set', 'Not set'), ('enabled', 'Enabled'), ('disabled', 'Disabled')], default='not_set', help_text='Stores whether the learner linked to this model has consented to have their information shared with the linked EnterpriseCustomer.', max_length=8),
        ),
    ]
